from __future__ import annotations

from digitalhub.entities.model._base.spec import ModelSpec, ModelValidator


class ModelSpecModel(ModelSpec):
    """
    ModelSpecModel specifications.
    """


class ModelValidatorModel(ModelValidator):
    """
    ModelValidatorModel validator.
    """
