from __future__ import annotations

from digitalhub.entities._base.entity.status import Status


class MaterialStatus(Status):
    """
    Material Status class.
    """

    def __init__(
        self,
        state: str,
        message: str | None = None,
        files: list[dict] | None = None,
    ) -> None:
        super().__init__(state, message)
        self.files = files if files is not None else []
