from __future__ import annotations

import typing

from digitalhub.entities.model._base.entity import Model

if typing.TYPE_CHECKING:
    from digitalhub.entities._base.entity.metadata import Metadata
    from digitalhub.entities.model.model.spec import ModelSpecModel
    from digitalhub.entities.model.model.status import ModelStatusModel


class ModelModel(Model):
    """
    ModelModel class.
    """

    def __init__(
        self,
        project: str,
        name: str,
        uuid: str,
        kind: str,
        metadata: Metadata,
        spec: ModelSpecModel,
        status: ModelStatusModel,
        user: str | None = None,
    ) -> None:
        super().__init__(project, name, uuid, kind, metadata, spec, status, user)

        self.spec: ModelSpecModel
        self.status: ModelStatusModel
